/*
 * Copyright 2014 OGIS-RI All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.co.ogis_ri.optaplanner.jobSolution;

import java.util.Date;

import jp.co.ogis_ri.optaplanner.jobSolution.domain.JobAssignment;
import jp.co.ogis_ri.optaplanner.jobSolution.domain.JobSolution;
import jp.co.ogis_ri.optaplanner.jobSolution.domain.People;

import org.optaplanner.core.api.domain.solution.Solution;
import org.optaplanner.core.api.solver.Solver;
import org.optaplanner.core.api.solver.SolverFactory;
import org.optaplanner.core.api.solver.event.BestSolutionChangedEvent;
import org.optaplanner.core.api.solver.event.SolverEventListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class App implements SolverEventListener {

	protected final transient Logger logger = LoggerFactory.getLogger(this
			.getClass());

	private long startDateTime;

	public static void main(final String[] args) {

		App app = new App();

		app.execute();
	}

	public void execute() {

		// Build the Solver
		SolverFactory solverFactory = SolverFactory
				.createFromXmlResource("jp/co/ogis_ri/optaplanner/jobSolution/solver/optaplannerConfig.xml");
		Solver solver = solverFactory.buildSolver();

		JobSolution unsolvedJobSolution = new JobSolutionGenerator()
				.createJobSolution();

		// リスナー登録
		solver.addEventListener(this);

		this.startDateTime = new Date().getTime();

		//
		solver.solve(unsolvedJobSolution);
		Solution solvedSolution = solver.getBestSolution();

		//
		this.printScore(solvedSolution);

		// Display the result
		String jobResult = toDisplayJobString((JobSolution) solvedSolution);

		System.out.println(jobResult);
	}

	public static String toDisplayJobString(final JobSolution jobSolution) {
		StringBuilder displayString = new StringBuilder();

		displayString.append("job\tday\tpeople\tskill\tattendance\n");

		for (JobAssignment jobAssignment : jobSolution.getJobAssignmentList()) {
			People people = jobAssignment.getPeople();
			displayString
					.append(jobAssignment.getJob())
					.append("\t")
					.append(jobAssignment.getJob().getDay())
					.append("\t")
					.append(people != null ? people : null)
					.append("\t")
					.append(people != null ? people.isSkill(jobAssignment
							.getJob()) : null)
					.append("\t")
					.append(people != null ? people.isAttendance(jobAssignment
							.getJob().getDay()) : null).append("\n");
		}
		return displayString.toString();
	}

	@Override
	public void bestSolutionChanged(final BestSolutionChangedEvent event) {
		this.printScore(event.getNewBestSolution());
	}

	private void printScore(final Solution solution) {

		this.logger.debug("TaskName:TaskName0 " + "ElapsedTime:"
				+ (new Date().getTime() - this.startDateTime) + "ms BestScore:"
				+ solution.getScore().toString());
	}
}
