/*
 * Copyright 2014 OGIS-RI All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.co.ogis_ri.optaplanner.jobSolution;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;

import jp.co.ogis_ri.optaplanner.jobSolution.domain.Day;
import jp.co.ogis_ri.optaplanner.jobSolution.domain.Job;
import jp.co.ogis_ri.optaplanner.jobSolution.domain.JobAssignment;
import jp.co.ogis_ri.optaplanner.jobSolution.domain.JobSolution;
import jp.co.ogis_ri.optaplanner.jobSolution.domain.People;

public class JobSolutionGenerator {

	public JobSolution createJobSolution() {

		String jobFileName = this.getClass().getClassLoader()
				.getResource("job.csv").getFile();
		String peopleFileName = this.getClass().getClassLoader()
				.getResource("people.csv").getFile();
		String skillFileName = this.getClass().getClassLoader()
				.getResource("skill.csv").getFile();
		String dayFileName = this.getClass().getClassLoader()
				.getResource("day.csv").getFile();
		String attendanceFileName = this.getClass().getClassLoader()
				.getResource("attendance.csv").getFile();

		//
		int id = 0;
		Hashtable<String, Day> dayTable = new Hashtable<String, Day>();
		Hashtable<String, Job> jobTable = new Hashtable<String, Job>();

		try {
			File jobFile = new File(jobFileName);
			File peopleFile = new File(peopleFileName);
			File skillFile = new File(skillFileName);
			File dayFile = new File(dayFileName);
			File attendanceFile = new File(attendanceFileName);

			BufferedReader jobBufferedReader;
			BufferedReader peopleBufferedReader;
			BufferedReader skillBufferedReader;
			BufferedReader dayBufferedReader;
			BufferedReader attendanceBufferedReader;

			jobBufferedReader = new BufferedReader(new FileReader(jobFile));
			peopleBufferedReader = new BufferedReader(
					new FileReader(peopleFile));

			// 人と仕事の個数を読み取る
			int jobCount = 0;
			while (jobBufferedReader.readLine() != null) {
				jobCount++;
			}
			jobCount = jobCount - 1;
			jobBufferedReader.close();

			int peopleCount = 0;
			while (peopleBufferedReader.readLine() != null) {
				peopleCount++;
			}
			peopleCount = peopleCount - 1;
			peopleBufferedReader.close();

			jobBufferedReader = new BufferedReader(new FileReader(jobFile));
			peopleBufferedReader = new BufferedReader(
					new FileReader(peopleFile));
			skillBufferedReader = new BufferedReader(new FileReader(skillFile));
			dayBufferedReader = new BufferedReader(new FileReader(dayFile));
			attendanceBufferedReader = new BufferedReader(new FileReader(
					attendanceFile));

			//
			int days = jobCount / 5;

			// 日付
			dayBufferedReader.readLine();
			List<Day> dayList = new ArrayList<Day>();
			for (int i = 0; i < days; i++) {
				String[] dayValueArray = dayBufferedReader.readLine()
						.split(",");

				String name = dayValueArray[0];
				int dayValue = Integer.parseInt(dayValueArray[1]);

				Day day = new Day(id++, name, dayValue);

				dayList.add(day);
				dayTable.put(dayValueArray[0], day);
			}

			// 仕事の一覧を作成、１日あたり５個の仕事になるように、割り当ては無しの状態で
			jobBufferedReader.readLine();
			List<JobAssignment> jobAssignmentList = new ArrayList<JobAssignment>();
			List<Job> jobList = new ArrayList<Job>();
			for (int i = 0; i < jobCount; i++) {
				String[] jobValueArray = jobBufferedReader.readLine()
						.split(",");

				String name = jobValueArray[0];
				Day day = dayTable.get(jobValueArray[1]);

				Job job = new Job(id++, name, day);
				jobList.add(job);
				jobTable.put(jobValueArray[0], job);

				JobAssignment jobAssignment = new JobAssignment(id++, name
						+ "Assignment", job);
				jobAssignmentList.add(jobAssignment);
			}

			// 人の一覧を作成
			peopleBufferedReader.readLine();
			skillBufferedReader.readLine();
			attendanceBufferedReader.readLine();
			List<People> peopleList = new ArrayList<People>();
			for (int i = 0; i < peopleCount; i++) {
				// スキルを読み込む
				String[] skillValueArray = skillBufferedReader.readLine()
						.split(",");
				HashMap<Job, Boolean> skillMap = new HashMap<Job, Boolean>();
				for (int j = 0; j < jobCount; j++) {

					Job job = jobList.get(j);
					boolean isSkill = skillValueArray[j + 1].equals("1");

					skillMap.put(job, isSkill);
				}

				// Attendanceを読み込み
				String[] attendanceValueArray = attendanceBufferedReader
						.readLine().split(",");
				HashMap<Day, Boolean> attendanceMap = new HashMap<Day, Boolean>();
				for (int j = 0; j < days; j++) {

					Day day = dayList.get(j);
					boolean attendance = Integer
							.parseInt(attendanceValueArray[j + 1]) == 1;

					attendanceMap.put(day, attendance);
				}

				// メンバー情報の読み込み
				String[] peopleValueArray = peopleBufferedReader.readLine()
						.split(",");

				String name = peopleValueArray[0];

				People people = new People(id++, name, skillMap, attendanceMap);

				peopleList.add(people);
			}

			jobBufferedReader.close();
			peopleBufferedReader.close();
			skillBufferedReader.close();
			dayBufferedReader.close();
			attendanceBufferedReader.close();

			return new JobSolution(id++, "JobSolution", peopleList, jobList,
					jobAssignmentList, dayList);

		} catch (IOException e) {
			throw new RuntimeException(e);
		}

	}
}
